package org.gcube.nlphub;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.gcube.nlphub.legacy.Constants;
import org.gcube.nlphub.legacy.JsonManager;
import org.gcube.nlphub.nlp.NlpAsyncNerRunner;
import org.gcube.nlphub.nlp.NlpParameter;
import org.gcube.nlphub.nlp.RunnerCommander;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * Servlet implementation class NLPHub
 */
@WebServlet(asyncSupported = true, name = "NLPServlet", urlPatterns = { "/nlphub-servlet" })
public class NLPHub extends HttpServlet {
	private Logger logger = Logger.getLogger(NLPHub.class.getSimpleName());
	private static final long serialVersionUID = 1L;
	public static final String service = "http://dataminer-prototypes.d4science.org/wps/";
	private String token; //= "df2cc5f5-63ee-48c1-b2a6-1210030c57b8-843339462";
	private boolean devMode = true;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public NLPHub() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		doWork(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		doWork(request, response);
	}

	private void doWork(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		if(request.getParameter("getInfo") != null) {
			getAlgorithmInfo(request, response);
		}
		else {
			runAlgorithms(request, response);
		}
	}
	
	private void getAlgorithmInfo(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		token = Constants.getToken(request, devMode);
		try {
			String dataMiner = request.getParameter("dataminer");
			String algId = request.getParameter("algId");
			HttpURLConnection connection = null;
			BufferedReader r = null;
			String algAbsrtact = "";
			PrintWriter writer = response.getWriter();
			response.setContentType("application/json;charset=UTF-8");
			try {
				String finalUrl = "https://" + dataMiner + NlpAsyncNerRunner.WPS_DESCRIBE_PROCESS_URL + "&gcube-token=" + token;
				finalUrl += "&lang=en-US&Identifier=" + algId;
				URL url = new URL(finalUrl);
				connection = (HttpURLConnection) url.openConnection();
				connection.setDoInput(true);
				connection.setDoOutput(true);
				connection.setUseCaches(false);
				connection.setRequestMethod("GET");
				r = new BufferedReader(new InputStreamReader(connection.getInputStream()));
				Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new InputSource(r));
				doc.getDocumentElement().normalize();
				
				NodeList nListInput = doc.getElementsByTagName("ows:Abstract");
				if(nListInput.getLength() <= 0) {
					algAbsrtact = "No description.";
					writer.println(new JsonManager().getErrorJsonResponse(algAbsrtact));
				}
				else {
					Node nodeAbstract = nListInput.item(0);
					algAbsrtact = nodeAbstract.getTextContent();
					writer.println(new JsonManager().getSuccessJsonResponse("" + algAbsrtact));
				}
				writer.close();

			} catch (Exception x) {
				logger.error(x.getLocalizedMessage());
				writer.println(new JsonManager().getErrorJsonResponse("" + x.getLocalizedMessage()));
				writer.close();
			} finally {
				try {
					if (r != null)
						r.close();
					if (connection != null)
						connection.disconnect();
				} catch (Exception e) {
					logger.error(e.getLocalizedMessage());
				}
			}
			
		} catch (Exception x) {
			logger.error(x.getLocalizedMessage());
		}		
	}
	
	private void runAlgorithms(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		token = Constants.getToken(request, devMode);
		try {
			String dataMiner = request.getParameter("dataminer");
			String[] algs = request.getParameter("algs").split(",");
			for(int i=0; i<algs.length; i++) {
				algs[i] = algs[i].trim();
			}
			
			
			RunnerCommander commander = new RunnerCommander(dataMiner, algs, request.getParameter("plink"), request.getParameter("annotations"), token,
					 response);
			commander.setSleepTime(100);
			commander.setMaxWaitingTime(90*1000);
			commander.startProcess();

		} catch (Exception x) {
			logger.error(x.getLocalizedMessage());
		}
		
	}
}
